/**
 * @class
 * Allows the retrieval and addition of comments and attachments.
 */
define("fraudCommons/widgets/CommentManager/CommentData", [
    'dojo/Deferred',
    'dojox/lang/functional',
    'dojo/_base/lang',
    'dojo/request',
    'fraudCommons/util/BaseDataMixin'
], function(Deferred, functional, lang, request, BaseDataMixin) {

	var JSON_COMMENTS = "src/fraudCommons/widgets/CommentManager/json/getCommentManager.json";

    var data = lang.mixin({}, new BaseDataMixin());

	data.setDebug = function(state) {
		this.debug = state;
	};

    data.hasAttachments = function(json, newComment, callback) {
        var promise = new Deferred();
        if(!this.debug) {
            var url = 'comments';
            var q = {
                action: 'hasAttachments'
            };
            var data;
            if(newComment) {
                q.isNewComment = true;
                functional.forIn(json, function(val, key) {
                    q[key] = val;
                });
            } else {
                q.id = json; //actually just a long
            }
            request.post(url, {
                query: q
            }).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
            });
        }
        return promise;
    };

	data.save = function(json, callback) {
	   var promise = new Deferred();
		if(!this.debug) {
			var url = 'comments';
            var d = {
                action: 'new'
            };
            functional.forIn(json, function(val, key) {
                d[key] = val;
            });
			request.post(url, {
                handleAs: "json",
                data: d
			}).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
			});
		}
		return promise;
	};

    data.update = function(json, callback) {
        var promise = new Deferred();
        if(!this.debug) {
            var url = 'comments';
            var d = {
                action: 'update'
            };
            functional.forIn(json, function(val, key) {
                d[key] = val;
            });
            request.post(url, {
                handleAs: "json",
                data: d
            }).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
            });
        }
        return promise;
    };
	
	/**
	 * Request all available comments from the midtier in json form
     * @param objectId
     * @param objectType
     * @param callback
	 */
	data.retrieve = function(objectId, objectType, objectName, callback) {
	   var promise = new Deferred();
		var url = 'comments';
		if(this.debug) {
			url = JSON_COMMENTS;
		}
		request.post(url, {
			handleAs: "json",
			query: {
				action: 'retrieve',
                objectId: objectId,
                objectType: objectType,
                objectName: objectName,
                noCacheFlag: new Date()
			}
		}).then(function(data) {
            if(callback) {
			      callback(data);
            }
            promise.resolve();
		});
		return promise;
	};

    data.deleteAttachments = function(attachments, commentId, callback) {
        var promise = new Deferred();
        var url = 'comments';
        if(this.debug) {
            url = JSON_COMMENTS;
        }
        request.post(url, {
            handleAs: "json",
            query: {
                action: 'deleteAttachments',
                attachments: JSON.stringify(attachments),
                commentId: commentId
            }
        }).then(function(data) {
            if(callback) {
                callback(data);
            }
            promise.resolve();
        });
        return promise;
    };

    data.deleteComment = function(id, callback) {
        var promise = new Deferred();
        var url = 'comments';
        if(this.debug) {
            url = JSON_COMMENTS;
        }
        request.post(url, {
            handleAs: "json",
            query: {
                action: 'deleteComment',
                id: id
            }
        }).then(function(data) {
            if(callback) {
                callback(data);
            }
            promise.resolve();
        });
        return promise;
    };
		
	return data;
});